import argparse
import asyncio
import dataclasses
import json
import logging

import httpx

from aioruuvigateway.api import get_gateway_history_data
from aioruuvigateway.models import TagData

log = logging.getLogger(__name__)


def json_default(o):
    if isinstance(o, bytes):
        return o.hex()
    raise TypeError(f"Object {o!r} is not JSON serializable")


def dump_tag_json(tag: TagData, *, parse: bool) -> str:
    data = dataclasses.asdict(tag)
    if parse:
        data["parsed"] = dataclasses.asdict(tag.parse_announcement())
    return json.dumps(
        data,
        ensure_ascii=False,
        default=json_default,
    )


async def run(
    host: str,
    token: str,
    interval: int,
    parse: bool = False,
    output_json: bool = False,
) -> None:
    async with httpx.AsyncClient() as client:
        last_tag_datas: dict[str, TagData] = {}
        while True:
            data = await get_gateway_history_data(
                client,
                host=host,
                bearer_token=token,
            )
            n_new = 0
            for tag in data.tags:
                if (
                    tag.mac not in last_tag_datas
                    or last_tag_datas[tag.mac].data != tag.data
                ):
                    try:
                        if output_json:
                            print(dump_tag_json(tag, parse=parse))
                        else:
                            print(tag)
                            if parse:
                                print("  ", tag.parse_announcement())
                    except Exception:
                        log.exception("Error printing tag %s", tag.mac)
                    last_tag_datas[tag.mac] = tag
                    n_new += 1
            if not n_new:
                log.info("No new data")
            await asyncio.sleep(interval)


def main() -> None:
    ap = argparse.ArgumentParser(
        description="Log Ruuvi Gateway history data",
    )
    ap.add_argument("--host", required=True)
    ap.add_argument("--token", required=True)
    ap.add_argument("--interval", type=int, default=5)
    ap.add_argument("--json", action="store_true", help="Output JSON")
    ap.add_argument("--parse", action="store_true", help="Parse advertisement data")
    args = ap.parse_args()
    logging.basicConfig(
        level=logging.INFO,
        datefmt="%Y-%m-%d %H:%M:%S",
        format="%(asctime)s %(levelname)s %(message)s",
    )
    asyncio.run(
        run(
            host=args.host,
            token=args.token,
            interval=args.interval,
            parse=args.parse,
            output_json=args.json,
        )
    )


if __name__ == "__main__":
    main()



from __future__ import annotations

import httpx

from aioruuvigateway.excs import CannotConnect, InvalidAuth
from aioruuvigateway.models import HistoryResponse


async def get_gateway_history_data(
    client: httpx.AsyncClient,
    host: str,
    bearer_token: str,
    timeout: int | None = None,
) -> HistoryResponse:
    try:
        resp = await client.get(
            url=f"http://{host}/history",
            headers={
                "Authorization": f"Bearer {bearer_token}",
            },
            timeout=(timeout if timeout else httpx.USE_CLIENT_DEFAULT),
        )
        if resp.status_code == 200:
            return HistoryResponse.from_gateway_history_json(resp.json())
        resp.raise_for_status()
    except httpx.HTTPStatusError as err:
        if err.response.status_code == 401:
            raise InvalidAuth from err
        raise CannotConnect from err
    except Exception as err:  # pragma: no cover
        raise CannotConnect from err
    raise NotImplementedError("This should not happen")



from collections import Counter
from pathlib import Path

import httpx
import pytest

from aioruuvigateway.api import get_gateway_history_data
from aioruuvigateway.excs import InvalidAuth

example_path = Path(__file__).parent / "example.json"


@pytest.mark.asyncio
async def test_library(httpx_mock):
    httpx_mock.add_response(
        url="http://192.168.1.202/history",
        content=example_path.read_bytes(),
        headers={"Content-Type": "application/json"},
    )
    async with httpx.AsyncClient() as client:
        history = await get_gateway_history_data(
            client=client,
            host="192.168.1.202",
            bearer_token="bear, a scary bear",
        )
    assert history.gw_mac_suffix == "EE:FF"
    manufacturers = Counter()
    for tag in history.tags:
        assert tag.datetime.year == 2022
        ann = tag.parse_announcement()
        print(ann)
        manufacturers.update(ann.manufacturer_data.keys())
    assert manufacturers == {
        0x0499: 2,  # Two Ruuvitags
        0x012D: 1,  # One Sony
        0x004C: 1,  # One Apple
    }


@pytest.mark.asyncio
async def test_auth(httpx_mock):
    httpx_mock.add_response(
        url="http://192.168.1.202/history",
        status_code=401,
    )
    async with httpx.AsyncClient() as client:
        with pytest.raises(InvalidAuth):
            await get_gateway_history_data(
                client=client,
                host="192.168.1.202",
                bearer_token="not good at all",
            )



from __future__ import annotations

import dataclasses
import datetime

from bluetooth_data_tools import BLEGAPAdvertisement, parse_advertisement_data


class TimestampConversionMixin:
    timestamp: int

    @property
    def datetime(self) -> datetime.datetime:
        return datetime.datetime.utcfromtimestamp(self.timestamp)


@dataclasses.dataclass()
class TagData(TimestampConversionMixin):
    mac: str  # AA:BB:CC:DD:EE:FF
    rssi: int
    timestamp: int
    data: bytes

    def parse_announcement(self) -> BLEGAPAdvertisement:
        return parse_advertisement_data([self.data])

    @classmethod
    def from_gateway_history_json_tag(cls, mac: str, data: dict) -> TagData:
        return cls(
            mac=mac,
            rssi=int(data["rssi"]),
            timestamp=int(data["timestamp"]),
            data=bytes.fromhex(data["data"]),
        )


@dataclasses.dataclass()
class HistoryResponse(TimestampConversionMixin):
    timestamp: int
    gw_mac: str
    tags: list[TagData]
    coordinates: str = ""

    @property
    def gw_mac_suffix(self) -> str:
        return self.gw_mac[-5:].upper()

    @classmethod
    def from_gateway_history_json(cls, data: dict) -> HistoryResponse:
        data = data["data"]
        tags = [
            TagData.from_gateway_history_json_tag(mac=mac, data=tag_data)
            for mac, tag_data in data.get("tags", {}).items()
        ]
        return HistoryResponse(
            timestamp=int(data["timestamp"]),
            gw_mac=data["gw_mac"],
            coordinates=data.get("coordinates", ""),
            tags=tags,
        )
